-- Copyright (C) 2003 Roger Browne. May be freely distributed under
-- the terms of the Eiffel Forum License version 2; see file LICENSE.

class FACT
	-- A single fact extracted from the factz file.

creation {FACTZ_FILE}

	make

feature {NONE}
	-- creation

	make is
		do
			create lines.make(1, 0)
		end

feature {FACTZ_FILE}
	-- commands

	set_date(fact_date: STRING) is
		require
			nonvoid_fact_date: fact_date /= void
			nonempty_fact_date: not fact_date.is_empty
		do
			date := fact_date
		ensure
			date: date = fact_date
		end
	
	set_deleted_by(later_fact: FACT) is
		-- This FACT has been deleted by a later FACT.
		require
			later_fact: later_fact /= void
		do
			has_been_deleted := true
			later_linked_fact := later_fact
		ensure
			has_been_deleted: has_been_deleted
			later_linked_fact: later_linked_fact = later_fact
		end
	
	set_deletes(earlier_fact: FACT) is
		-- This FACT deletes an earlier FACT.
		require
			earlier_fact: earlier_fact /= void
		do
			is_a_deletion := true
			earlier_linked_fact := earlier_fact
		ensure
			is_a_deletion: is_a_deletion
			earlier_linked_fact: earlier_linked_fact = earlier_fact
		end
	
	set_modified_by(later_fact: FACT) is
		-- This FACT has been modified by a later FACT.
		require
			later_fact: later_fact /= void
		do
			has_been_modified := true
			later_linked_fact := later_fact
		ensure
			has_been_modified: has_been_modified
			later_linked_fact: later_linked_fact = later_fact
		end
	
	set_modifies(earlier_fact: FACT) is
		-- This FACT modifies an earlier FACT.
		require
			earlier_fact: earlier_fact /= void
		do
			is_a_modification := true
			earlier_linked_fact := earlier_fact
		ensure
			is_a_modification: is_a_modification
			earlier_linked_fact: earlier_linked_fact = earlier_fact
		end

	add_line(fact_line: STRING) is
		require
			nonvoid_fact_line: fact_line /= void
		do
			lines.force(fact_line, lines.upper + 1)
		ensure
			set: lines.item(lines.upper) = fact_line
			added: lines.count = 1 + old lines.count
		end

feature {ANY}
	-- queries

	date: STRING
	
	deletes: FACT is
		-- Earlier FACT deleted by this one
		require
			is_a_deletion
		do
			result := earlier_fact_date
		ensure
			date: result = earlier_fact_date
		end
	
	deleted_by: FACT is
		-- Later FACT that deleted this one
		require
			has_been_deleted
		do
			result := later_fact_date
		ensure
			date: result = later_fact_date
		end
		
	has_been_deleted: BOOLEAN
		-- Has this FACT been deleted by a later FACT?
	
	has_been_modified: BOOLEAN
		-- Has this FACT been modified by a later FACT?
	
	is_a_deletion: BOOLEAN
		-- Does this FACT denote the deletion of an earlier FACT?
		
	is_a_modification: BOOLEAN
		-- Does this FACT denote a modification of an earlier FACT?

	line(n: INTEGER): STRING is
		require
			n: n >= 1 and n <= line_count
		do
			result := lines.item(n)
		end

	line_count: INTEGER is
		do
			result := lines.count
		end
	
	modifies: FACT is
		-- Date of earlier FACT modified by this one
		require
			is_a_modification
		do
			result := earlier_fact_date
		ensure
			date: result = earlier_fact_date
		end
	
	modified_by: STRING is
		-- Later FACT that modified this one
		require
			has_been_modified
		do
			result := later_fact_date
		ensure
			date: result = later_fact_date
		end

feature {NONE}
	-- queries

	lines: ARRAY[STRING]
		
	earlier_linked_fact: FACT
		-- Earlier FACT linked to this one (because this one deletes or modifies it).
	
	later_linked_fact: FACT
		-- Later FACT linked to this one (because it deletes or modifies this one).

invariant

	lines: lines /= void
	not_both_1: not (is_a_deletion and is_a_modification)
	not_both_2: not (has_been_deleted and has_been_modified)
	earlier_linked_fact: (is_a_deletion or is_a_modification) = (earlier_linked_fact /= void)
	later_linked_fact: (has_been_deleted or has_been_modified) = (later_linked_fact /= void)
	
	-- Undecided: whether a "deletion" can itself be "deleted", thereby "undeleting" the original FACT.

end
