-- Copyright (C) 2003 Roger Browne. May be freely distributed under
-- the terms of the Eiffel Forum License version 2; see file LICENSE.

class FACTZ

creation make

feature {ANY}
	-- creation

	make is
		local
			command: STRING
		do
			create factz_file
			create cgi.make
			command := cgi.lookup("submit")
			if command = void then
				new_fact(void)
			elseif command.is_equal("Delete") then
				delete_fact(
					cgi.lookup("Confirmed"),
					cgi.lookup("Factdate"),
					cgi.lookup("Searchtext")
				)
			elseif command.is_equal("Modify") then
				modify_fact(
					cgi.lookup("Confirmed"),
					cgi.lookup("Factdate"),
					cgi.lookup("Searchtext"),
					cgi.lookup("Facttext")
				)
			elseif command.is_equal("Save") then
				save_new_fact(
					cgi.lookup("Facttext"),
					cgi.lookup("Searchtext")
				)
			elseif command.is_equal("Search") then
				search_factz(cgi.lookup("Searchtext"))
			elseif command.is_equal("Show All") then
				new_fact(void)
			elseif command.is_equal("View") then
				view_fact(
					cgi.lookup("Fact"),
					cgi.lookup("Searchtext")
				)
			else
				error("Unrecognised value for 'submit' CGI parameter: '" + command + "'")
			end
		rescue
			-- Handle any exceptions that occur before the HTTP header has been sent.
			-- Other exceptions should be handled by class WEBPAGE.
			report_exception
		end

feature {NONE}
	-- commands
	-- Each of these commands implements an operation selected by the user.

	delete_fact(confirmed: STRING; fact_date: STRING; searchtext: STRING) is
		do
			if confirmed = void then
				-- Ask user for confirmation.
				create webpage.make
				webpage.set_fact(factz_file.fact_with_date(fact_date))
				webpage.set_searchtext(searchtext)
				scan_factz(searchtext)
				webpage.emit_confirm_delete_page
			elseif confirmed.is_equal("Yes") then
				if fact_date /= void then
					factz_file.delete_fact(fact_date)
					new_fact(searchtext)
				else
					error("CGI Error: 'Delete' was requested, but the date of the FACT to be deleted is missing.")
				end
			else  -- "No" was clicked
				view_fact(fact_date, searchtext)
			end
		end

	modify_fact(confirmed: STRING; fact_date: STRING; searchtext: STRING; fact_text: STRING) is
		do
			if confirmed = void then
				-- Allow user to modify the fact
				create webpage.make
				webpage.set_fact(factz_file.fact_with_date(fact_date))
				webpage.set_searchtext(searchtext)
				scan_factz(searchtext)
				webpage.emit_modify_fact_page
			else
				if confirmed.is_equal("Save") then
					factz_file.modify_fact(fact_date, fact_text)
					view_fact(factz_file.datestamp, searchtext)
				else
					view_fact(fact_date, searchtext)
				end
			end
		end

	new_fact(searchtext: STRING) is
		do
			create webpage.make
			scan_factz(searchtext)
			webpage.emit_new_fact_page
		end

	save_new_fact(fact_text: STRING; searchtext: STRING) is
		do
			if fact_text /= void and then fact_text.count > 0 then
				factz_file.append_new_fact(fact_text)
				view_fact(factz_file.datestamp, searchtext)
			else
				new_fact(searchtext)
			end
		end

	search_factz(searchtext: STRING) is
		do
			new_fact(searchtext)
		end

	view_fact(fact_date: STRING; searchtext: STRING) is
		do
			create webpage.make
			scan_factz(searchtext)
			webpage.set_fact(factz_file.fact_with_date(fact_date))
			webpage.emit_view_fact_page
		end

feature {NONE}
	-- commands (subcommands called by the above features)

	error(message: STRING) is
		require
			message: message /= void and then not message.is_empty
		do
			create webpage.make
			webpage.set_error_message(message)
			webpage.emit_error_page
		end
		
	report_exception is
		local
			exception: expanded EXCEPTIONS
			code: INTEGER
		do
			print("Content-type: text/plain%N%N")
			print("SYSTEM ERROR%N%N")
			print("An Eiffel exception was raised on the webserver.%N%N")
			if exception.is_developer_exception then
				print("Developer exception")
				if exception.developer_exception_name /= void and then not exception.developer_exception_name.is_empty then
					print(": ")
					print(exception.developer_exception_name)
				end
				print('.')
			else
				code := exception.exception
				if exception.assertion_violation then
					print("Assertion violation: ")
				elseif exception.is_signal then
					print("Signal: ")
				else
					print("Unknown exception type: ")
				end
				print(code)
				print(" (")
				if code = exception.Check_instruction then
					print("Check_instruction")
				elseif code = exception.Class_invariant then
					print("Class_invariant")
				elseif code = exception.Developer_exception then
					print("Developer_exception")
				elseif code = exception.Incorrect_inspect_value then
					print("Incorrect_inspect_value")
				elseif code = exception.Loop_invariant then
					print("Loop_invariant")
				elseif code = exception.Loop_variant then
					print("Loop_variant")
				elseif code = exception.No_more_memory then
					print("No_more_memory")
				elseif code = exception.Postcondition then
					print("Postcondition")
				elseif code = exception.Precondition then
					print("Precondition")
				elseif code = exception.Routine_failure then
					print("Routine_failure")
				elseif code = exception.Os_signal then
					print("Os_signal")
					print(' ')
					print(exception.signal_number)
				elseif code = exception.Void_attached_to_expanded then
					print("Void_attached_to_expanded")
				elseif code = exception.Void_call_target then
					print("Void_call_target")
				elseif code = exception.System_level_type_error then
					print("System_level_type_error")
				else
					print("Unknown exception type")
				end
				print(")%N")
			end
		end

	scan_factz(searchtext: STRING) is
		local
			fact: FACT
			factz_used, i: INTEGER
		do
			webpage.set_searchtext(searchtext)
			from
				i := factz_file.fact_count
				factz_used := 0
			variant i
			until i = 0 or factz_used = factz_per_page loop
				fact := factz_file.fact(i)
				if fact.has_been_deleted or fact.has_been_modified or fact.is_a_deletion then
					-- Don't display the fact
				elseif searchtext = void or else searchtext.is_empty or else matching_fact(fact, searchtext) then
					webpage.add_list_item(fact)
					factz_used := factz_used + 1
				end
				i := i - 1
			end
		end

feature {NONE}
	-- queries

	cgi: CGI  -- CGI parser by Dustin Sallings
	
	factz_file: FACTZ_FILE
		-- Handles all interaction with the data file that stores the FACTZ.
	
	factz_per_page: INTEGER is 10  -- maximum number of search results of recent factz to display per page

	webpage: WEBPAGE

	matching_fact(fact: FACT; searchtext: STRING): BOOLEAN is
		-- Does the fact's date or content match 'searchtext'?
		require
			fact: fact /= void
			searchtext: searchtext /= void and then not searchtext.is_empty
		local
			line: INTEGER
		do
			if fact.date.has_substring(searchtext) then
				result := true
			else
				from
					line := 1
				until line > fact.line_count or result loop
					result := fact.line(line).has_substring(searchtext) 
					line := line + 1
				end
			end
		end

end
